@@ -315,7 +315,7 @@ static void Rcairo_setup_font(CairoGDDesc* xd, R_GE_gcontext *gc) {
 				strcat(spec, specs[i - 5]);
 				Rcairo_set_font(i, spec);
 			} else
-				Rcairo_set_font(i, spec);
+				Rcairo_set_font(i, gc->fontfamily);
 			if (Rcairo_fonts[i].family)
 				free(Rcairo_fonts[i].family);
 			Rcairo_fonts[i].family = strdup(gc->fontfamily);
@@ -658,7 +658,7 @@ static rc_text_shape shared_text_shape;
 /* FIXME: this is not thread safe. Threads are not supported in R
    anyway, but just saying... This will be used across Cairo
    devices, but keeping one buffer reduces allocations. */   
-static rc_text_shape *init_text_shape() {
+static rc_text_shape *init_text_shape(void) {
 	shared_text_shape.x = 0.0;
 	shared_text_shape.y = 0.0;
 	shared_text_shape.glyphs = 0;
@@ -783,7 +783,7 @@ static rc_text_shape *c_setup_glyphs(CairoGDDesc *xd, R_GE_gcontext *gc, const c
 	ulen = str2utf16(str, strlen(str), &text, encoding) / sizeof(UChar); /* str2utf16 returns bytes, need chars */
 	ubidi_setPara(bidi, text, ulen, UBIDI_DEFAULT_LTR, NULL, &err);
 	if (U_FAILURE(err))
-		Rf_error("Unable to compute UBiDi for string '%'", str);
+		Rf_error("Unable to compute UBiDi for string '%s'", str);
 
 	rc_text_shape *ts = init_text_shape();
 	int i = xd->fontface - 1;
@@ -1023,7 +1023,7 @@ Rboolean CairoGD_Open(NewDevDesc *dd, CairoGDDesc *xd,  const char *type, int co
 		xd->cb->width = w; xd->cb->height = h;
 		xd->cb->flags|=CDF_NOZERO;
 		if (!strcmp(type,"pdf"))
-			xd->cb = Rcairo_new_pdf_backend(xd->cb, conn, file, w, h);
+			xd->cb = Rcairo_new_pdf_backend(xd->cb, conn, file, w, h, aux);
 		else if (!strcmp(type,"ps") || !strcmp(type,"postscript"))
 			xd->cb = Rcairo_new_ps_backend(xd->cb, conn, file, w, h);
 		else if (!strcmp(type,"svg"))
@@ -1549,7 +1549,7 @@ void Rcairo_backend_init_surface(Rcairo_backend *be) {
 }
 
 /* add any new back-ends to this list */
-void Rcairo_register_builtin_backends() {
+void Rcairo_register_builtin_backends(void) {
 	if (RcairoBackendDef_image) Rcairo_register_backend(RcairoBackendDef_image);
 	if (RcairoBackendDef_pdf) Rcairo_register_backend(RcairoBackendDef_pdf);
 	if (RcairoBackendDef_ps) Rcairo_register_backend(RcairoBackendDef_ps);
@@ -1585,7 +1585,7 @@ void attribute_visible R_init_Cairo(DllInfo *dll) {
 }
 
 /* called on load */
-SEXP Rcairo_initialize() {
+SEXP Rcairo_initialize(void) {
 	Rcairo_register_builtin_backends();
 	return R_NilValue;
 }
